﻿// <copyright file="NameSearch.cs" company="Edge Extensions Project">
// Copyright (c) 2009 All Rights Reserved
// </copyright>
// <author>Kevin Nessland</author>
// <email>kevinnessland@gmail.com</email>
// <date>2009-07-08</date>
// <summary>Assists with the searching of files by file name.</summary>
namespace Edge.Extensions.Helpers
{
    using System;
    using System.Collections;
    using System.IO;

    /// <summary>
    /// Provides methods for searching files based upon their file name.
    /// </summary>
    public class NameSearch
    {
        /// <summary>
        /// Used to store the list of found file names.
        /// </summary>
        private ArrayList names;

        /// <summary>
        /// Determines whether or not to recursively search for extensions.
        /// </summary>
        private bool recursive;

        /// <summary>
        /// Initializes a new instance of the NameSearch class.
        /// </summary>
        public NameSearch()
        {
            this.names = ArrayList.Synchronized(new ArrayList());

            this.recursive = true;
        }

        /// <summary>
        /// Gets the list of names used in the search.
        /// </summary>
        public ArrayList SearchNames
        {
            get { return this.names; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to recurse sub-directories.
        /// </summary>
        public bool Recursive
        {
            get { return this.recursive; }
            set { this.recursive = value; }
        }

        /// <summary>
        /// Performs a name search in the specified directory path.
        /// </summary>
        /// <param name="path">Directory path to search.</param>
        /// <returns>Results of the search.</returns>
        public FileInfo[] Search(string path)
        {
            DirectoryInfo root = new DirectoryInfo(path);

            ArrayList subFiles = new ArrayList();

            try
            {
                foreach (FileInfo file in root.GetFiles())
                {
                    if (this.names.Contains(file.Name))
                    {
                        subFiles.Add(file);
                    }
                }

                if (this.recursive)
                {
                    foreach (DirectoryInfo directory in root.GetDirectories())
                    {
                        subFiles.AddRange(this.Search(directory.FullName));
                    }
                }
            }
            catch (UnauthorizedAccessException)
            {
                // might be nice to log this, or something ...
            }

            return (FileInfo[])subFiles.ToArray(typeof(FileInfo));
        }
    }
}
